import subprocess  # nosec
import sys

from django.contrib.staticfiles.management.commands import collectstatic

from django_esm.conf import get_settings


class Command(collectstatic.Command):
    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            "--no-esm",
            "--noesm",
            action="store_true",
            help="Do not collect ES modules before collecting static files.",
        )

    def handle(self, **options):
        if not options["no_esm"]:
            subprocess.check_call(  # nosec
                [
                    "npx",
                    "--yes",
                    "esimport",
                    get_settings().PACKAGE_DIR,
                    get_settings().STATIC_DIR,
                ],
                stdout=sys.stdout,
                stderr=sys.stderr,
            )
        super().handle(**options)



from django.conf import settings

__all__ = ["get_settings"]


def get_settings():
    return type(
        "Settings",
        (),
        {
            "PACKAGE_DIR": "",
            "STATIC_DIR": "",
            "STATIC_PREFIX": "",
            **getattr(settings, "ESM", {}),
        },
    )



import json
import pathlib

from django import template
from django.conf import settings
from django.contrib.staticfiles.storage import staticfiles_storage
from django.utils.safestring import mark_safe

from .. import conf

register = template.Library()

importmap_json = {}


def _resolve_importmap_urls(raw_importmap):
    full_importmap = {
        "imports": {},
        "integrity": {},
    }
    for entry_pointy, filename in raw_importmap["imports"].items():
        static_url = staticfiles_storage.url(
            str(pathlib.Path(conf.get_settings().STATIC_PREFIX) / filename)
        )
        full_importmap["imports"][entry_pointy] = static_url
        full_importmap["integrity"][static_url] = raw_importmap["integrity"][filename]
    return json.dumps(
        full_importmap,
        indent=2 if settings.DEBUG else None,
        separators=None if settings.DEBUG else (",", ":"),
    )


@register.simple_tag
def importmap():
    global importmap_json
    if not importmap_json or settings.DEBUG:
        with (
            pathlib.Path(conf.get_settings().STATIC_DIR) / "importmap.json"
        ).open() as f:
            raw_importmap = json.load(f)
            importmap_json = _resolve_importmap_urls(raw_importmap)
    return mark_safe(importmap_json)  # nosec



import pathlib

from django.conf import settings
from django.core.checks import Error, Tags, Warning, register

from . import conf

__all__ = ["check_esm_settings"]


@register(Tags.staticfiles)
def check_esm_settings(app_configs, **kwargs):
    errors = []
    if not conf.get_settings().PACKAGE_DIR:
        errors.append(
            Error(
                'Setting ESM["PACKAGE_DIR"] is not configured.',
                hint=(
                    'ESM["PACKAGE_DIR"] must be an absolute path or pathlib.Path object.'
                ),
                id="esm.E001",
            )
        )

    if not pathlib.Path(conf.get_settings().PACKAGE_DIR).is_absolute():
        errors.append(
            Error(
                'ESM["PACKAGE_DIR"] is a relative path.',
                hint=(
                    'ESM["PACKAGE_DIR"] must be an absolute path or pathlib.Path object.'
                ),
                id="esm.E002",
            )
        )

    if not conf.get_settings().STATIC_DIR:
        errors.append(
            Error(
                'Setting ESM["STATIC_DIR"] is not configured.',
                hint=(
                    'ESM["STATIC_DIR"] must be an absolute path or pathlib.Path object.'
                ),
                id="esm.E003",
            )
        )

    if not pathlib.Path(conf.get_settings().STATIC_DIR).is_absolute():
        errors.append(
            Error(
                'ESM["STATIC_DIR"] is a relative path.',
                hint=(
                    'ESM["STATIC_DIR"] must be an absolute path or pathlib.Path object.'
                ),
                id="esm.E004",
            )
        )

    if not conf.get_settings().STATIC_PREFIX:
        errors.append(
            Error(
                'Setting ESM["STATIC_PREFIX"] is not configured.',
                hint=(
                    'ESM["STATIC_PREFIX"] must be an absolute path or pathlib.Path object.'
                ),
                id="esm.E005",
            )
        )

    if (
        conf.get_settings().STATIC_PREFIX,
        conf.get_settings().STATIC_DIR,
    ) not in settings.STATICFILES_DIRS:
        errors.append(
            Error(
                '\'(ESM["STATIC_PREFIX"], ESM["STATIC_DIR"]),\' must be in STATICFILES_DIRS.',
                id="esm.E006",
            )
        )

    if not (pathlib.Path(conf.get_settings().PACKAGE_DIR) / "package.json").exists():
        errors.append(
            Error(
                f"package.json file not found in: {conf.get_settings().PACKAGE_DIR}",
                hint='Make sure check your ESM["PACKAGE_DIR"] setting.',
                id="esm.E007",
            )
        )

    if not (pathlib.Path(conf.get_settings().STATIC_DIR) / "importmap.json").exists():
        errors.append(
            Warning(
                f"importmap.json file not found in: {conf.get_settings().STATIC_DIR}",
                hint=(
                    'Make sure check your ESM["STATIC_DIR"] setting and to run the "esm" management command to generate the importmap.json file.'
                ),
                id="esm.W001",
            )
        )

    return errors


@register(Tags.staticfiles, deploy=True)
def check_deployment(app_configs, **kwargs):
    errors = []

    if not (pathlib.Path(conf.get_settings().STATIC_DIR) / "importmap.json").exists():
        errors.append(
            Error(
                f"importmap.json file not found in: {conf.get_settings().STATIC_DIR}",
                hint=(
                    'Make sure check your ESM["STATIC_DIR"] setting and to run the "esm" management command to generate the importmap.json file.'
                ),
                id="esm.E008",
            )
        )

    return errors



import subprocess  # nosec
import sys

from django.core.management import BaseCommand

from django_esm.conf import get_settings


class Command(BaseCommand):
    """Collect ES modules from the package directory and generate importmap."""

    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument(
            "-w",
            "--watch",
            action="store_true",
            help="Watch for changes in the package directory and re-run collect files.",
        )

    def handle(self, *args, **options):
        subprocess.check_call(  # nosec
            (
                [
                    "npx",
                    "--yes",
                    "esimport",
                    get_settings().PACKAGE_DIR,
                    get_settings().STATIC_DIR,
                ]
                + (["--watch"] if options["watch"] else [])
            ),
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
